<?php

namespace App\Http\Controllers;

use App\ApiClasses\Error;
use App\ApiClasses\Success;
use App\Enums\DomainRequestStatus;
use App\Models\SuperAdmin\DomainRequest;
use App\Models\SuperAdmin\Plan;
use App\Models\User;
use App\Services\CommonService\TenantService;
use Constants;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;

class DomainRequestController extends Controller
{
    /**
     * @var TenantService
     */
    protected $tenantService;

    /**
     * DomainRequestController constructor.
     *
     * @param TenantService $tenantService
     */
    public function __construct(TenantService $tenantService)
    {
        $this->tenantService = $tenantService;
    }

    public function index()
    {
        return view('superAdmin.domainRequest.index');
    }

    public function indexAjax(Request $request)
    {
        try {
            $columns = [
                1 => 'id',
                2 => 'user',
                3 => 'name',
                4 => 'created_at',
                5 => 'status',
            ];

            $query = DomainRequest::query();

            $limit = $request->input('length');
            $start = $request->input('start');
            $order = $columns[$request->input('order.0.column')];
            $dir = $request->input('order.0.dir');

            $totalData = $query->count();

            if ($order == 'id') {
                $order = 'domain_requests.id';
                $query->orderBy($order, $dir);
            }

            // Base query with all necessary joins
            $baseQuery = $query->select(
                'domain_requests.*',
                'user.first_name',
                'user.last_name',
                'user.email',
                'tenant_codes.code as tenant_code'
            )
                ->leftJoin('users as user', 'domain_requests.user_id', '=', 'user.id')
                ->leftJoin('tenants', function ($join) {
                    $join->where('tenants.id', '=', DB::raw('user.email'));
                })
                ->leftJoin('tenant_codes', 'tenant_codes.tenant_id', '=', 'tenants.id');

            if (empty($request->input('search.value'))) {
                $domainRequests = clone $baseQuery;
                $domainRequests = $domainRequests->offset($start)
                    ->limit($limit)
                    ->get();
            } else {
                $search = $request->input('search.value');
                $domainRequests = clone $baseQuery;
                $domainRequests = $domainRequests->where(function ($query) use ($search) {
                    $query->where('domain_requests.id', 'like', '%' . $search . '%')
                        ->orWhere('domain_requests.user_id', 'like', '%' . $search . '%')
                        ->orWhere('user.first_name', 'like', '%' . $search . '%')
                        ->orWhere('user.last_name', 'like', '%' . $search . '%')
                        ->orWhere('user.email', 'like', '%' . $search . '%')
                        ->orWhere('tenant_codes.code', 'like', '%' . $search . '%');
                })
                    ->offset($start)
                    ->limit($limit)
                    ->get();
            }

            $totalFiltered = $domainRequests->count();

            $data = [];

            if (!empty($domainRequests)) {
                foreach ($domainRequests as $domainRequest) {
                    $nestedData['id'] = $domainRequest->id;
                    $nestedData['user'] = $domainRequest->user_id;
                    $nestedData['name'] = $domainRequest->name;
                    $nestedData['created_at'] = $domainRequest->created_at->format(Constants::DateTimeFormat);
                    $nestedData['status'] = $domainRequest->status;
                    $nestedData['tenant_code'] = $domainRequest->tenant_code ?? 'Not created yet';

                    // User
                    $nestedData['user_name'] = $domainRequest->user->getFullName();
                    $nestedData['user_email'] = $domainRequest->user->email;
                    $nestedData['user_initials'] = $domainRequest->user->getInitials();
                    $nestedData['user_profile_image'] = $domainRequest->user->profile_picture != null ? tenant_asset(Constants::BaseFolderEmployeeProfileWithSlash . $domainRequest->user->profile_picture) : null;

                    $data[] = $nestedData;
                }
            }

            return response()->json([
                'draw' => intval($request->input('draw')),
                'recordsTotal' => $totalData,
                'recordsFiltered' => $totalFiltered,
                'data' => $data,
            ]);
        } catch (Exception $e) {
            Log::error($e->getMessage());
            return Error::response('Something went wrong');
        }
    }

    public function getByIdAjax($id)
    {
        $domainRequest = DomainRequest::findOrFail($id);
        if (!$domainRequest) {
            return Error::response('Domain request not found');
        }
        $response = [
            'id' => $domainRequest->id,
            'userName' => $domainRequest->user->getFullName(),
            'userEmail' => $domainRequest->user->email,
            'name' => $domainRequest->name,
            'status' => $domainRequest->status,
            'createdAt' => $domainRequest->created_at->format(Constants::DateTimeFormat),
        ];
        return Success::response($response);
    }

    public function actionAjax(Request $request)
    {
        if (env('APP_DEMO')) {
            return redirect()->back()->with('error', 'This feature is disabled in the demo.');
        }

        $validated = $request->validate([
            'id' => 'required|exists:domain_requests,id',
            'status' => 'required|in:approved,rejected',
            'adminNotes' => 'nullable|string',
        ]);

        try {
            $domainRequest = DomainRequest::findOrFail($validated['id']);
            
            // If rejecting, update status immediately
            if ($validated['status'] == 'rejected') {
                $domainRequest->status = DomainRequestStatus::from($validated['status']);
                $domainRequest->reject_reason = $validated['adminNotes'] ?? null;
                $domainRequest->save();
                return back()->with('success', 'Domain request rejected successfully.');
            }
            
            // For approval, first try to create the tenant
            if ($validated['status'] == 'approved') {
                // Store notes but don't change status yet
                $domainRequest->approve_reason = $validated['adminNotes'] ?? null;
                $domainRequest->save();
                
                // Try to create tenant
                $tenant = $this->createTenant($domainRequest);
                
                // Only update status to approved if tenant creation was successful
                if ($tenant) {
                    $domainRequest->status = DomainRequestStatus::APPROVED;
                    $domainRequest->save();
                    return back()->with('success', 'Domain request approved and tenant created successfully.');
                } else {
                    return back()->with('error', 'Failed to create tenant. Please check logs and try again.');
                }
            }

            return back()->with('success', 'Domain request ' . $validated['status'] . ' successfully.');
        } catch (Exception $e) {
            Log::error($e->getMessage());
            return back()->with('error', 'Something went wrong. Please try again.');
        }
    }

    /**
     * Create a new tenant for an approved domain request
     * 
     * @param DomainRequest $domainRequest
     * @return \App\Models\Tenant|null
     */
    private function createTenant(DomainRequest $domainRequest)
    {
        try {
            $user = User::findOrFail($domainRequest->user_id);
            $plan = Plan::findOrFail($user->plan_id);
            
            $data = [
                'name' => $domainRequest->name ?? $user->first_name . ' ' . $user->last_name,
            ];
            
            // Call tenant service to create tenant
            $tenant = $this->tenantService->createTenant($data, $user, $plan);
            
            if (!$tenant) {
                Log::error('Tenant creation failed for domain request', ['domain_request_id' => $domainRequest->id]);
                return null;
            }
            
            Log::info('Tenant created successfully', ['tenant_id' => $tenant->id, 'domain_request_id' => $domainRequest->id]);
            return $tenant;
        } catch (Exception $e) {
            Log::error('Error creating tenant: ' . $e->getMessage());
            Log::error($e->getTraceAsString());
            return null;
        }
    }
}